#!/bin/sh
# This script must be able to run on busybox-powered devices, so uses POSIX shell instead of bash.

DISTRIBUTION_PACKAGE_DIR="$(dirname "$0")"
TAR_FILE="${DISTRIBUTION_PACKAGE_DIR}/mediaserver.tar.gz"

PRODUCT_NAME="metavms"

# TARGET_DIR is set either from NX_VMS_INSTALL_DIR environment variable or read from the file
# in the Server data directory.
TARGET_DIR=""

BACKUP_CONFIG_DIR=""
BACKUP_PLUGINS_DIR=""

get_data_dir()
{
    flash_card_mount_point="$(df | grep "/dev/localstorage" | awk '{print $6}')"
    if [ "${flash_card_mount_point}" = "" ]; then
        echo "Cannot find the mount point of the SD card." >&2
        exit 1
    fi
    echo -n "${flash_card_mount_point}/${PRODUCT_NAME}.data"
}

get_static_data_dir()
{
    app_path="$1"

    app_storage_mount_point="$( \
        ( df "${app_path}" 2>/dev/null || echo "" ) | tail -1 | awk '{print $6}' )"
    if [ -z "${app_storage_mount_point}" ]; then
        echo "Cannot find the mount point of the internal storage." >&2
        exit 1
    fi
    echo -n "${app_storage_mount_point}/${PRODUCT_NAME}.license"
}

get_module_dir()
{
    echo -n "${TARGET_DIR}/mediaserver"
}

get_config_dir()
{
    echo -n "$(get_module_dir)/etc"
}

get_server_plugins_dir()
{
    echo -n "$(get_module_dir)/bin/plugins"
}

get_bundled_plugins_file()
{
    echo -n "$(get_module_dir)/bundled_plugins.txt"
}

set_target_dir()
{
    data_dir="$(get_data_dir)"

    if [ -f "${data_dir}/vadp_package_location.txt" ]; then
        TARGET_DIR="$(cat "$(get_data_dir)/vadp_package_location.txt")"
    else
        cat <<EOF
===================================================================================================
ATTENTION: Current Server installation is not found; if it is not the first-time installation, some
misconfiguration must have been occurred.
===================================================================================================
EOF
        TARGET_DIR="${DISTRIBUTION_PACKAGE_DIR}"
    fi

    if [ ! -d "${TARGET_DIR}" ]; then
        echo "FAILED: Cannot find the installed Server." >&2
        exit 1
    fi
}

remove_bundled_plugins()
{
    server_plugins_dir="$(get_server_plugins_dir)"
    if [ ! -d "${server_plugins_dir}" ]; then
        return
    fi

    bundled_plugins_file="$(get_bundled_plugins_file)"
    if [ ! -f "${bundled_plugins_file}" ]; then
        cat <<EOF
===================================================================================================
ATTENTION: Some misconfiguration might have occurred: the bundled plugin list file was not found:
${bundled_plugins_file}
===================================================================================================
EOF
        return
    fi
    # Delete each file or directory listed in the file.
    xargs -I{} rm -Rf "${server_plugins_dir}/{}" < "${bundled_plugins_file}"
}

create_bundled_plugins_list()
{
    bundled_plugins_file="$(get_bundled_plugins_file)"
    server_plugins_dir="$(get_server_plugins_dir)"

    echo "# List of the plugins supplied with the VMS Server package." > "${bundled_plugins_file}"
    ( #< Use a subshell to avoid changing the current directory.
        cd "${server_plugins_dir}"
        for plugin in *; do
            echo "${plugin}" >> "${bundled_plugins_file}"
        done
    )
}

back_up_necessary_files()
{
    # Back up the config if needed.
    config_dir="$(get_config_dir)"
    if [ -d "${config_dir}" ]; then
        # "/tmp" has a very little space on these devices, so use the Server data directory instead.
        BACKUP_CONFIG_DIR="$(mktemp -t -d -p "$(get_data_dir)/tmp")"
        cp -r "${config_dir}/"* "${BACKUP_CONFIG_DIR}"
    fi

    # Back up the plugins if needed.
    server_plugins_dir="$(get_server_plugins_dir)"
    if [ -n "$(ls -A "${server_plugins_dir}")" ]; then
        BACKUP_PLUGINS_DIR="$(mktemp -t -d -p "$(get_data_dir)/tmp")"
        cp -r "${server_plugins_dir}/"* "${BACKUP_PLUGINS_DIR}/"
    fi
}

restore_backup_files()
{
    # Restore the config if needed.
    if [ -d "${BACKUP_CONFIG_DIR}" ]; then
        config_dir="$(get_config_dir)"
        cp -r "${BACKUP_CONFIG_DIR}/"* "${config_dir}"
        rm -Rf "${BACKUP_CONFIG_DIR}"
    fi

    if [ -d "${BACKUP_PLUGINS_DIR}" ]; then
        server_plugins_dir="$(get_server_plugins_dir)"
        # Restore a plugin only if there is no such plugin in the target directory (i.e. no such
        # plugin is bundled with the new version), hence the "-n" flag for the "mv" command.
        mv -n "${BACKUP_PLUGINS_DIR}"/* "${server_plugins_dir}"
        rm -Rf "${BACKUP_PLUGINS_DIR}"
    fi
}

upgrade_vms()
{
    module_dir="$(get_module_dir)"

    control_script="${module_dir}/bin/mediaserver.sh"

    if [ -f "${control_script}" ]; then
        "${control_script}" stop
    fi

    remove_bundled_plugins
    back_up_necessary_files

    # Remove the old installation.
    rm -rf "${module_dir}"
    if [ "${DISTRIBUTION_PACKAGE_DIR}" != "${TARGET_DIR}" ]; then
        rm "${TARGET_DIR}"/*.sh "${TARGET_DIR}/vadp.xml"
        cp "${DISTRIBUTION_PACKAGE_DIR}"/*.sh "${TARGET_DIR}"
        cp "${DISTRIBUTION_PACKAGE_DIR}/vadp.xml" "${TARGET_DIR}"
    fi

    # Unpack the distribution.
    tar xfv "${TAR_FILE}" -C "${TARGET_DIR}"

    create_bundled_plugins_list

    config_dir="${module_dir}/etc"
    mediaserver_data_dir="$(get_data_dir)"
    static_data_dir="$(get_static_data_dir "${TARGET_DIR}")"

    # Create data directories and restore the config from the backup.
    mkdir -p "${static_data_dir}"
    mkdir -p "${mediaserver_data_dir}"
    echo "${TARGET_DIR}" > "${mediaserver_data_dir}/vadp_package_location.txt"

    restore_backup_files

    # Update the config.
    "${module_dir}/bin/upgrade_config.sh" \
        "${module_dir}/etc/mediaserver.conf" \
        "${module_dir}/share/configs/mediaserver.conf" \
        "${module_dir}/etc/mediaserver_conf_added_keys.txt" \
        "${module_dir}/bin/config_helper.sh"

    # Remove the distribution file if it is in the target directory.
    if [ "${DISTRIBUTION_PACKAGE_DIR}" == "${TARGET_DIR}" ]; then
        rm -f "${TAR_FILE}"
    fi

    if [ -z "${NX_NO_AUTOSTART+x}" ]; then #< Start the Server only if NX_NO_AUTOSTART is not set.
        "${control_script}" start
    fi
}

main()
{
    set_target_dir

    if [ "$1" != "" ]; then
        upgrade_vms >>"$1" 2>&1
    else
        upgrade_vms 2>&1
    fi
}

main "$@"
